//********************** 3601test.c ****************************
//* This file contains demo (test) program for CPS serial port
//* controlled power supplies.
//* Written by Alexander Kostromitin
//* Copyright (C) CPS, 2006
//**************************************************************

#ifndef WINVER                          // Specifies that the minimum required platform is Windows Vista.
#define WINVER 0x0600           // Change this to the appropriate value to target other versions of Windows.
#endif

#ifndef _WIN32_WINNT            // Specifies that the minimum required platform is Windows Vista.
#define _WIN32_WINNT 0x0600     // Change this to the appropriate value to target other versions of Windows.
#endif

#ifndef _WIN32_WINDOWS          // Specifies that the minimum required platform is Windows 98.
#define _WIN32_WINDOWS 0x0410 // Change this to the appropriate value to target Windows Me or later.
#endif

#ifndef _WIN32_IE                       // Specifies that the minimum required platform is Internet Explorer 7.0.
#define _WIN32_IE 0x0700        // Change this to the appropriate value to target other versions of IE.
#endif
#define WIN32_LEAN_AND_MEAN             // Exclude rarely-used stuff from Windows headers
// Windows Header Files:
#include <windows.h>

// C RunTime Header Files
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <tchar.h>


#include <time.h>
#include <string.h>
#include <stdio.h>
#include <windows.h>
#include <commdlg.h>    // includes common dialog functionality
#include <commctrl.h>
#include <dlgs.h>       // includes common dialog template defines
#include <cderr.h>      // includes the common dialog error codes
#include <io.h>
#include <math.h>


#include "..\serial\serial.h"
#include "c3601.h"
#include "c3601win.h"
#include "3601test.h"

#define USERMODE    1       // disable some "for internal use" functions


//////////////////////// Types //////////////////////
enum { okay, acc_temp, hv_temp, level } alarmtype;

////////////////////// Function prototypes //////////////////////////

int APIENTRY MainWndProc(HWND hDlg, int uMsg, WPARAM wParam, LPARAM lParam);
BOOL InitApplication(void);
BOOL InitInstance(void);
BOOL APIENTRY startdlg(HWND hDlg, UINT message, UINT wParam, LONG lParam);
BOOL CALLBACK helpdialog(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK alarmdialog(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

void showsts(void);
void filter(double *val, double sample);
double detcorona(double i);
void alarm(enum alarmtype atype);
void disableall(void);

void curvtofile(void);
void logprintf(char *fmt, ... );
void readini(void);
void readinipar(char *buf, char *name, double *par);

void dlgprintf(HWND hDlg, int id, char *fmt, ... );
int dlgscanf(HWND hDlg, int id, char *fmt, ... );
int error(char *fmt, ... );

void help(HELPINFO *info, HWND win);
char *gettip(int id);



////////////////////// globals ///////////////////////

int debugmode;                      // debug mode: if disabled, will try to
                                    //   restore communication instead error messages
int port;                           // 0 -> COM port 1 ... 3 -> COM port 4
int connectok_a, connectok_e,
    connectok_f, remoteok;          // connection ok flags
int stop;                           // force to stop, no attempts to retry
int status;
int boardtype = 3601;
double vset_a;
double vset_e;
double vset_f;
double curtemp;
double coronalevel;

int pause, usewd = 1;

int cycltestkey = 0;                    // Use cyclic test

///////////// device specific parameters //////////////
static double vscale_a_abs =   30000;      // absolut v scale, setpoint, Volts
static double vscale_a_s   =   30000;      // output v scale, setpoint, Volts
static double voffset_a_s  =   0;          // output v offset, setpoint, Volts

static double vscale_a     =   30000;      // output v scale, meter, Volts
static double voffset_a    =   0;          // output v offset, meter, Volts
static double iscale_a     =   500e-6;     // output i scale, meter, Ampers
static double ioffset_a    =   0;          // output i offset, meter, Ampers

static double vscale_e_abs =   10000;      // absolut v scale, setpoint, Volts
static double vscale_e_s   =   10000;      // output v scale, setpoint, Volts
static double voffset_e_s  =   0;          // output v offset, setpoint, Volts
static double vscale_e_s_r =   1;          // output r=v/i ratio

static double vscale_e     =   10000;      // output v scale, meter, Volts
static double voffset_e    =   0;          // output v offset, meter, Volts
static double iscale_e     =   500e-6;     // output i scale, meter, Ampers
static double ioffset_e    =   0;          // output i offset, meter, Ampers

static double iscale_f_abs =   5;          // absolut i scale, setpoint, Ampers
static double iscale_f_s   =   5;          // output i scale, setpoint, Ampers
static double ioffset_f_s  =   0;          // output i offset, setpoint, Ampers

static double iscale_f     =   5;          // output i scale, meter, Ampers
static double ioffset_f    =   0;          // output i offset, meter, Ampers
static double vscale_f     =   5;          // output v scale, meter, Volts
static double voffset_f    =   0;          // output v offset, meter, Volts

static double shutdowntemp =   60;         // shutdown temperature, C
static int    shutdowndelay =  10;         // shutdown delay, s
static int    useshutdown  =   0;          // use shutdown if !=0

static int    PASSIVE_BIAS =      0;       // bias output regulator is active (0) or passive (1)
static int    VOLTAGE_CONTROL_F = 0;       // filament control uses current (0) or voltage (1)
static int    EXTRA_PARAMS =      0;       // show extra parameters in control window (1)
static int    RESISTOR_CONTROL  = 0;       // bias control uses voltage reg. (0) or variable resistor (1)
static int    SEPARATED_FEEDBACK= 1;       // accelerator has a single HV connetion (0) or separated Out and Feedback (1)

// Limits
static double MinimumAcceleratorSet = 0;
static double MaximumAcceleratorSet = 50000;

static double MinimumBiasSet        = 500;
static double MaximumBiasSet        = 5000;

static double MinimumFilamentSet    = 0;
static double MaximumFilamentSet    = 2.200;



#define minmax(x,l,r)   (((x)<(l))?(l):((x)>(r))?(r):(x))

double fltrv, fltri, fltt;          // filtered values: valid only if connectok
double fltav, fltai;                // filtered values, adjusted to scale
int fltdelay;                       // delay (in ms) before filter starts
int lastextmeter;                   // last use of external meter: 'v'-voltage; 'i'-current

#define TICK        200              // timer tick period, ms

#define LOGTICKS    (10000/TICK)    // number of tick in log period
//#define LOGTICKS    (30000/TICK)    // number of tick in log period

HINSTANCE MainInst;                 // the current instance
HWND      MainWin;                  // hWnd of main window

char *AppName = "360x_control";
char *AppHdr = "360x Control Panel";

int helpact;                        // context help active

int ntbfracts_a;                    // number of slider's positions
int ntbfracts_e;                    // number of slider's positions
int ntbfracts_f;                    // number of slider's positions

char workdir[256];

int errorsts;                       // error status, returned by last call of a C3601_Get/Setxxx()
int devicests;                      // device status, returned by last call of C3601_GetStatusFlags()
int faultsts;                       // device fault status, returned by last call of C3601_GetFaultFlags()


#ifndef _MSC_VER
#pragma argsused
#endif

int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
//int APIENTRY _tWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
//  LPSTR lpCmdLine, int nCmdShow)
{
  char *p;
  int err;

  // get work directory
  GetModuleFileName(0, workdir, 255);   // analog of the argv[0]
  p = strrchr(workdir, '\\');
  if(p)
    *p = 0;

  err = C3601_DLLOpen(workdir);
  if(err)
    {
    error("DLL Open error %x", err);
    return 0;
    }

  // read configuration data
  readini();

  MainInst = hInstance;
  if(InitApplication())
    InitInstance();

  return(0);
}


BOOL InitApplication(void)
{
  WNDCLASS wc;

  // default values
  memset(&wc, 0, sizeof(wc));

  wc.style = 0;
  wc.lpfnWndProc = (WNDPROC)MainWndProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = MainInst;
  wc.hIcon = LoadIcon(MainInst, "AIcon");
  wc.hCursor = 0;
  wc.hbrBackground = 0;
  wc.lpszMenuName = 0;
  wc.lpszClassName = AppName;
  return(RegisterClass(&wc));
}


BOOL InitInstance(void)
{
  // MSC: add comctl32.lib to ProjectSettings / Link / Library
  InitCommonControls();
  MainWin = CreateWindow(
      AppName,
      AppHdr,
      WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT,
      CW_USEDEFAULT,
      100,
      100,
      NULL,
      NULL,
      MainInst,
      NULL);
  if(!MainWin) return(FALSE);
  ShowWindow(MainWin, SW_MINIMIZE);
  DialogBox(MainInst, "Edit", MainWin, startdlg);
  return(TRUE);
}


#ifndef _MSC_VER
#pragma argsused
#endif

int APIENTRY MainWndProc(HWND hDlg, int uMsg, WPARAM wParam, LPARAM lParam)
{
  switch(uMsg)
    {
    case WM_CREATE:
      break;
    case WM_DESTROY:
      PostQuitMessage(0);
      break;
    default: return(DefWindowProc(hDlg, uMsg, wParam, lParam));
    }
  return(0);
}


#ifndef _MSC_VER
#pragma argsused
#endif

// Start dialog callback
BOOL APIENTRY startdlg(HWND hDlg, UINT message, UINT wParam, LONG lParam)
{
  static HWND tb_a, pb_a;
  static HWND tb_e, pb_e;
  static HWND tb_f, pb_f;
  static HWND pb_c;
  int tpos;
  int cmd, ver_a, ver_e, ver_f;
  static double vsout_a, vsout_e, vsout_f;
  static int stopdealy;
  static double v_e;

  switch(message)
    {
    case WM_INITDIALOG:         // init controls
      MainWin = hDlg;

      // set up default control's states and 3601 values
      SetWindowPos(MainWin, HWND_TOPMOST, 50, 50, 0, 0, SWP_NOSIZE);
      SetWindowPos(MainWin, HWND_NOTOPMOST, 50, 50, 0, 0, SWP_NOSIZE);
      SendMessage(hDlg, WM_SETICON, 0, (LPARAM)LoadIcon(MainInst, "AIcon"));

      SendDlgItemMessage(hDlg, IDC_3601, BM_SETCHECK, boardtype == 3601, 0);
      SendDlgItemMessage(hDlg, IDC_3602, BM_SETCHECK, boardtype == 3602, 0);
      SendDlgItemMessage(hDlg, IDC_3603, BM_SETCHECK, boardtype == 3603, 0);

      SendDlgItemMessage(hDlg, IDC_COM1, BM_SETCHECK, port == 0, 0);
      SendDlgItemMessage(hDlg, IDC_COM2, BM_SETCHECK, port == 1, 0);
      SendDlgItemMessage(hDlg, IDC_COM3, BM_SETCHECK, port == 2, 0);
      SendDlgItemMessage(hDlg, IDC_COM4, BM_SETCHECK, port == 3, 0);

      vset_a = MinimumAcceleratorSet;
      vset_e = MinimumBiasSet;
      vset_f = MinimumFilamentSet;

      SetTimer(hDlg, 1, TICK, 0);

      restart:                      // restart connection

      connectok_a = connectok_e = connectok_f = !C3601_OpenPort(port, DEF_BAUD_RATE);

      if(connectok_a)                 // try to communicate
        {
        ver_a = C3601_GetVersion(port, DEV_A);
        if(ver_a <= 0)
          connectok_a = ver_a = 0;
        }
      if(connectok_e)                 // try to communicate
        {
        ver_e = C3601_GetVersion(port, DEV_E);
        if(ver_e <= 0)
          connectok_e = ver_e = 0;
        }
      if(connectok_f)                 // try to communicate
        {
        ver_f = C3601_GetVersion(port, DEV_F);
        if(ver_f <= 0)
          connectok_f = ver_f = 0;
        }

      ntbfracts_a = vscale_a_abs / 500;     // 500V fixed clicks

      if(vscale_e_abs < 1500)
        ntbfracts_e = vscale_e_abs / 10;    // 10V fixed clicks
      else if(vscale_e_abs < 2500)
        ntbfracts_e = vscale_e_abs / 20;    // 20V fixed clicks
      else if(vscale_e_abs < 3500)
        ntbfracts_e = vscale_e_abs / 25;    // 25V fixed clicks
      else if(vscale_e_abs < 5000)
        ntbfracts_e = vscale_e_abs / 50;    // 50V fixed clicks
      else
        ntbfracts_e = vscale_e_abs / 100;   // 100V fixed clicks

      ntbfracts_f = iscale_f_abs / 0.1;     // 0.1A fixed clicks

      C3601_Reset(port, DEV_A, 0);
      C3601_Reset(port, DEV_F, 0);

      Sleep(1000);

      C3601_SetValue(port, DEV_A, 0, 0);
      C3601_SetValue(port, DEV_E, 0, 0);
      C3601_SetValue(port, DEV_F, 0, 0);

      tb_a = GetDlgItem(hDlg, IDC_V_SET_A_S);
      SendMessage(tb_a, TBM_SETRANGE, 1, MAKELONG(0, ntbfracts_a));
      pb_a = GetDlgItem(hDlg, IDC_SETMETER_A);
      SendMessage(pb_a, PBM_SETRANGE, 0, MAKELONG(0, 1000));

      tb_e = GetDlgItem(hDlg, IDC_V_SET_E_S);
      SendMessage(tb_e, TBM_SETRANGE, 1, MAKELONG(0, ntbfracts_e));
      pb_e = GetDlgItem(hDlg, IDC_SETMETER_E);
      SendMessage(pb_e, PBM_SETRANGE, 0, MAKELONG(0, 1000));

      tb_f = GetDlgItem(hDlg, IDC_V_SET_F_S);
      SendMessage(tb_f, TBM_SETRANGE, 1, MAKELONG(0, ntbfracts_f));
      pb_f = GetDlgItem(hDlg, IDC_SETMETER_F);
      SendMessage(pb_f, PBM_SETRANGE, 0, MAKELONG(0, 1000));

      pb_c = GetDlgItem(hDlg, IDC_CMETER);
      SendMessage(pb_c, PBM_SETRANGE, 0, MAKELONG(0, 1000));

      ShowWindow(GetDlgItem(hDlg, IDC_USE_A), USERMODE ? SW_HIDE : SW_SHOW);
      ShowWindow(GetDlgItem(hDlg, IDC_USE_E), USERMODE ? SW_HIDE : SW_SHOW);
      ShowWindow(GetDlgItem(hDlg, IDC_USE_F), USERMODE ? SW_HIDE : SW_SHOW);
      ShowWindow(GetDlgItem(hDlg, IDC_LOG), USERMODE ? SW_HIDE : SW_SHOW);

      ShowWindow(GetDlgItem(hDlg, IDC_ENABLE_E), PASSIVE_BIAS ? SW_HIDE : SW_SHOW);

      ShowWindow(GetDlgItem(hDlg, IDC_CALC_STATIC), !PASSIVE_BIAS ? SW_HIDE : SW_SHOW);
      ShowWindow(GetDlgItem(hDlg, IDC_V_EM_STATIC), !PASSIVE_BIAS ? SW_HIDE : SW_SHOW);
      ShowWindow(GetDlgItem(hDlg, IDC_I_EM_STATIC), !PASSIVE_BIAS ? SW_HIDE : SW_SHOW);
      ShowWindow(GetDlgItem(hDlg, IDC_V_EM), !PASSIVE_BIAS ? SW_HIDE : SW_SHOW);
      ShowWindow(GetDlgItem(hDlg, IDC_I_EM), !PASSIVE_BIAS ? SW_HIDE : SW_SHOW);

      ShowWindow(GetDlgItem(hDlg, IDC_R_F), EXTRA_PARAMS ? SW_SHOW : SW_HIDE);
      ShowWindow(GetDlgItem(hDlg, IDC_P_F), EXTRA_PARAMS ? SW_SHOW : SW_HIDE);
      ShowWindow(GetDlgItem(hDlg, IDC_T_F), EXTRA_PARAMS ? SW_SHOW : SW_HIDE);
      ShowWindow(GetDlgItem(hDlg, IDC_R_STATIC), EXTRA_PARAMS ? SW_SHOW : SW_HIDE);
      ShowWindow(GetDlgItem(hDlg, IDC_P_STATIC), EXTRA_PARAMS ? SW_SHOW : SW_HIDE);
      ShowWindow(GetDlgItem(hDlg, IDC_T_STATIC), EXTRA_PARAMS ? SW_SHOW : SW_HIDE);

      ShowWindow(GetDlgItem(hDlg, IDC_V_EM), SEPARATED_FEEDBACK ? SW_HIDE : SW_SHOW);
      ShowWindow(GetDlgItem(hDlg, IDC_V_EM_STATIC), SEPARATED_FEEDBACK ? SW_HIDE : SW_SHOW);

      if(VOLTAGE_CONTROL_F)
        SetWindowText(GetDlgItem(hDlg, IDC_SET_F_STATIC), "Voltage to set");
      else
        SetWindowText(GetDlgItem(hDlg, IDC_SET_F_STATIC), "Current to set");
      if(RESISTOR_CONTROL)
        SetWindowText(GetDlgItem(hDlg, IDC_SET_E_STATIC), "Resistor, kOhm");
      else
        SetWindowText(GetDlgItem(hDlg, IDC_SET_E_STATIC), "Voltage to set");

      dlgprintf(hDlg, IDC_V_SET_A_M, "%6.0f", vset_a);
      dlgprintf(hDlg, IDC_V_SET_E_M, "%6.0f", vset_e);
      dlgprintf(hDlg, IDC_V_SET_F_M, "%6.3f", vset_f);

      if(connectok_a)
        {
        C3601_SetWD(port, DEV_A, usewd ? 1000 : 0);
        }
      if(connectok_e)
        {
        C3601_SetWD(port, DEV_E, usewd ? 1000 : 0);
        }
      if(connectok_f)
        {
        C3601_SetWD(port, DEV_F, usewd ? 1000 : 0);
        }
      SetDlgItemText(hDlg, IDC_CONSTAT_A,  connectok_a ? "Connected" : "no comm");
      SetDlgItemText(hDlg, IDC_CONSTAT_E,  connectok_e ? "Connected" : "no comm");
      SetDlgItemText(hDlg, IDC_CONSTAT_F,  connectok_f ? "Connected" : "no comm");

      dlgprintf(hDlg, IDC_REVISION, "%04x:%04x:%04x", ver_a, ver_e, ver_f);

      tpos = vset_a * ntbfracts_a / vscale_a_abs;
      SendMessage(tb_a, TBM_SETPOS, 1, tpos);
      tpos = vset_e * ntbfracts_e / vscale_e_abs;
      SendMessage(tb_e, TBM_SETPOS, 1, tpos);
      tpos = vset_f * ntbfracts_f / iscale_f_abs;
      SendMessage(tb_f, TBM_SETPOS, 1, tpos);

      //if(readprofile(ports))
      //  defaultprofile();
      //else
      //  w_profile = r_profile;
      //writeprofile(port);
      break;
    case WM_NOTIFY:
      {
      NMHDR *hdr = (NMHDR *)lParam;

      switch(hdr -> code)
        {
        case UDN_DELTAPOS:          // setpoint up/down
          {
          NM_UPDOWN *ud = (NM_UPDOWN *)lParam;
          switch(wParam)
            {
            case IDC_V_SET_A_UD:
              vset_a -= ud -> iDelta * 5;
              vset_a = minmax(vset_a, MinimumAcceleratorSet, MaximumAcceleratorSet);
              dlgprintf(hDlg, IDC_V_SET_A_M, "%6.0f", vset_a);
              tpos = vset_a * ntbfracts_a / vscale_a_abs;
              SendMessage(tb_a, TBM_SETPOS, 1, tpos);
              break;
            case IDC_V_SET_E_UD:
              vset_e -= ud -> iDelta / 10.0;
              vset_e = minmax(vset_e, MinimumBiasSet, MaximumBiasSet);
              dlgprintf(hDlg, IDC_V_SET_E_M, "%6.1f", vset_e);
              tpos = vset_e * ntbfracts_e / vscale_e_abs;
              SendMessage(tb_e, TBM_SETPOS, 1, tpos);
              break;
            case IDC_V_SET_F_UD:
              vset_f -= ud -> iDelta / 1000.0;
              vset_f = minmax(vset_f, MinimumFilamentSet, MaximumFilamentSet);
              dlgprintf(hDlg, IDC_V_SET_F_M, "%6.3f", vset_f);
              tpos = vset_f * ntbfracts_f / vscale_f;
              SendMessage(tb_f, TBM_SETPOS, 1, tpos);
              break;
            }
          }
        }
      break;
      }
    case WM_HSCROLL:                // setpoint scroll
      {
      double v;
      if((HWND)lParam == tb_a)
        {
        if(LOWORD(wParam)!=SB_THUMBTRACK)
          {
          tpos = vset_a * ntbfracts_a / vscale_a_abs;
          SendMessage(tb_a, TBM_SETPOS, 1, tpos);
          SetFocus(pb_c);
          break;
          }
        tpos = SendMessage(tb_a, TBM_GETPOS, 0, 0);
        v = tpos * vscale_a_abs / ntbfracts_a;
        if(v != vset_a)
          {
          vset_a = minmax(v, MinimumAcceleratorSet, MaximumAcceleratorSet);
          if(v != vset_a)
            {
            tpos = vset_a * ntbfracts_a / vscale_a_abs;
            SendMessage(tb_a, TBM_SETPOS, 1, tpos);
            }
          dlgprintf(hDlg, IDC_V_SET_A_M, "%6.0f", vset_a);
          }
        }
      else if((HWND)lParam == tb_e)
        {
        if(LOWORD(wParam)!=SB_THUMBTRACK)
          {
          tpos = vset_e * ntbfracts_e / vscale_e_abs;
          SendMessage(tb_e, TBM_SETPOS, 1, tpos);
          SetFocus(pb_c);
          break;
          }
        tpos = SendMessage(tb_e, TBM_GETPOS, 0, 0);
        v = tpos * vscale_e_abs / ntbfracts_e;
        if(v != vset_e)
          {
          vset_e = minmax(v, MinimumBiasSet, MaximumBiasSet);
          if(v != vset_e)
            {
            tpos = vset_e * ntbfracts_e / vscale_e_abs;
            SendMessage(tb_e, TBM_SETPOS, 1, tpos);
            }
          dlgprintf(hDlg, IDC_V_SET_E_M, "%6.1f", vset_e);
          }
        }
      else if((HWND)lParam == tb_f)
        {
        if(LOWORD(wParam)!=SB_THUMBTRACK)
          {
          tpos = vset_f * ntbfracts_f / iscale_f_abs;
          SendMessage(tb_f, TBM_SETPOS, 1, tpos);
          SetFocus(pb_c);
          break;
          }
        tpos = SendMessage(tb_f, TBM_GETPOS, 0, 0);
        v = tpos * iscale_f_abs / ntbfracts_f;
        if(v != vset_f)
          {
          vset_f = minmax(v, MinimumFilamentSet, MaximumFilamentSet);
          if(v != vset_f)
            {
            tpos = vset_f * ntbfracts_f / iscale_f_abs;
            SendMessage(tb_f, TBM_SETPOS, 1, tpos);
            }
          dlgprintf(hDlg, IDC_V_SET_F_M, "%6.3f", vset_f);
          }
        }
      break;
      }
    case WM_MOUSEWHEEL:    // 0x020A
      {
      int dy = (short)HIWORD(wParam);
      POINT mp;
      HWND cw;

      if(dy != 0)
        {
        int id;

        mp.x = LOWORD(lParam);
        mp.y = HIWORD(lParam);
        if(ScreenToClient(hDlg, &mp))
          {
          NM_UPDOWN ud;

          cw = ChildWindowFromPoint(hDlg, mp);
          id = GetWindowLong(cw, GWL_ID);
          if(id == IDC_ACC)
            cw = tb_a;
          else if(id == IDC_BIAS)
            cw = tb_e;
          else if(id == IDC_FIL)
            cw = tb_f;
          else
            break;
          if(cw == tb_a || cw == tb_e || cw == tb_f)
            {
            if(cw == tb_a)
              wParam = IDC_V_SET_A_UD;
            else if(cw == tb_e)
              wParam = IDC_V_SET_E_UD;
            else
              wParam = IDC_V_SET_F_UD;
            ud.hdr.code = UDN_DELTAPOS;
            ud.iDelta = dy > 0 ? -1 : 1;
            lParam = (LPARAM)&ud;
            SendMessage(hDlg, WM_NOTIFY, wParam, lParam);
            }
          }
        }
      break;
      }
    case WM_SYSCOMMAND:
      if(wParam == SC_CONTEXTHELP)
        helpact = 1;
      if(wParam == SC_CLOSE)
        goto close;
      return(FALSE);

    case WM_HELP:
      {
      static HELPINFO inf, *f1;
      f1 = (HELPINFO *)lParam;
      if(!helpact)                      // F1 key pressed
        {
        RECT wr;
        GetWindowRect(hDlg, &wr);
        inf.MousePos.x = (wr.left + wr.right) / 2 - 100;
        inf.MousePos.y = (wr.top + wr.bottom) / 2 - 100;
        inf.iCtrlId = IDC_CPS;
        //GetWindowOrgEx(hDlg, &inf.MousePos);// = helppoint;
        help(&inf, hDlg);
        }
      else
        {
        helpact = 0;
        help(f1, hDlg);                 // use current id
        }
      break;
      }
    case WM_COMMAND:
      cmd = LOWORD(wParam);
      switch(cmd)
        {
        case IDC_CONSTOP:           // stop connection
          if(connectok_a)
            {
            C3601_SetValue(port, DEV_A, 0, 0);
            }
          if(connectok_e)
            {
            C3601_SetValue(port, DEV_E, 0, 0);
            }
          if(connectok_f)
            {
            C3601_SetValue(port, DEV_F, 0, 0);
            }
          C3601_ClosePort(port);
          connectok_a = connectok_e = connectok_f = 0;
          stop = 1;
          SetDlgItemText(hDlg, IDC_CONSTAT_A,  connectok_a ? "Connected" : "stop");
          SetDlgItemText(hDlg, IDC_CONSTAT_E,  connectok_e ? "Connected" : "stop");
          SetDlgItemText(hDlg, IDC_CONSTAT_F,  connectok_f ? "Connected" : "stop");
          break;

        case IDC_CONRESTART:        // restart connection
          if(connectok_a && connectok_e && connectok_f)
            break;
          if(SendDlgItemMessage(hDlg, IDC_COM1, BM_GETCHECK, 0, 0))
            port = 0;
          else if(SendDlgItemMessage(hDlg, IDC_COM2, BM_GETCHECK, 0, 0))
            port = 1;
          else if(SendDlgItemMessage(hDlg, IDC_COM3, BM_GETCHECK, 0, 0))
            port = 2;
          else
            port = 3;

          stop = 0;
          goto restart;

        case IDC_3601:
        case IDC_3602:
        case IDC_3603:
          {
          if(SendDlgItemMessage(hDlg, IDC_3601, BM_GETCHECK, 0, 0))
            boardtype = 3601;
          if(SendDlgItemMessage(hDlg, IDC_3602, BM_GETCHECK, 0, 0))
            boardtype = 3602;
          if(SendDlgItemMessage(hDlg, IDC_3603, BM_GETCHECK, 0, 0))
            boardtype = 3603;
          readini();
          goto restart;
          }
        case IDC_CLOSE:
          if(GetKeyState(VK_CONTROL)&0x8000)    // Ctrl key
            {
            readini();
            goto restart;
            }
        close:                                  // close all and quit
          if(connectok_a)
            {
            C3601_SetValue(port, DEV_A, 0, 0);
            }
          if(connectok_e)
            {
            C3601_SetValue(port, DEV_E, 0, 0);
            }
          if(connectok_f)
            {
            C3601_SetValue(port, DEV_F, 0, 0);
            }
          C3601_ClosePort(port);
          KillTimer(hDlg, 1);
          PostQuitMessage(0);
        default: return(FALSE);
        }
      break;
    case WM_TIMER:                  // timer event
      {
      int alarmlevel, pos, enab_a, enab_e, enab_f;
      int use_a, use_e, use_f;
      double v, i, v_a, i_a, t_a = 0, t_e = 0, t_f = 0;
      double i_e;
      static double vavr, iavr, tavr;
      static int tickcnt;
      static int newlog = 1;
      static HBITMAP rbmp, gbmp;
      static oldalarmlevel = -1;

      alarmlevel = C3601_GetAlarm(port);


      //SendDlgItemMessage(hDlg, IDC_V_SET_A_S, WM_KILLFOCUS, 0, 0);

      if(alarmlevel != oldalarmlevel)
        {
        if(!rbmp)
          rbmp = LoadBitmap(MainInst, "RlampBmp");
        if(!gbmp)
          gbmp = LoadBitmap(MainInst, "GlampBmp");
        SendDlgItemMessage(hDlg, IDC_ALARM, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM)(alarmlevel ? rbmp : gbmp));
        oldalarmlevel = alarmlevel;
        }

      #if(USERMODE)
        use_a = use_e = use_f = 1;
      #else
        use_a = SendDlgItemMessage(hDlg, IDC_USE_A, BM_GETCHECK, 0, 0);
        use_e = SendDlgItemMessage(hDlg, IDC_USE_E, BM_GETCHECK, 0, 0);
        use_f = SendDlgItemMessage(hDlg, IDC_USE_F, BM_GETCHECK, 0, 0);
      #endif

      if(use_e)
        {
        v_e = C3601_GetVoltage(port, DEV_E, &errorsts) * vscale_e - voffset_e;
        dlgprintf(hDlg, IDC_V_OUT_E, "%9.3lf", v_e);      // V

        i = C3601_GetCurrent(port, DEV_E, &errorsts) * iscale_e - ioffset_e;
        dlgprintf(hDlg, IDC_I_OUT_E, "%9.3lf", i * 1e6);  // uA

        if(PASSIVE_BIAS)
          {
          i_e = i;
          }

        t_e = C3601_GetTempearture(port, DEV_E, &errorsts);
        }

      if(use_a)
        {
        v_a = C3601_GetVoltage(port, DEV_A, &errorsts) * vscale_a - voffset_a;
        dlgprintf(hDlg, IDC_V_OUT_A, "%9.3lf", v_a);      // V

        i_a = i = C3601_GetCurrent(port, DEV_A, &errorsts) * iscale_a - ioffset_a;
        dlgprintf(hDlg, IDC_I_OUT_A, "%9.3lf", i * 1e6);  // uA

        coronalevel = detcorona(i) * 1e9;                 // full scale (1000 units) is 1 uA
        pos = coronalevel;
        SendMessage(pb_c, PBM_SETPOS, pos, 0);
        t_a = C3601_GetTempearture(port, DEV_A, &errorsts);
        }

      if(use_f)
        {
        double r, rf, t;

        v = C3601_GetVoltage(port, DEV_F, &errorsts) * vscale_f - voffset_f;
        dlgprintf(hDlg, IDC_V_OUT_F, "%5.3lf", v);        // V

        i = C3601_GetCurrent(port, DEV_F, &errorsts) * iscale_f - ioffset_f;
        dlgprintf(hDlg, IDC_I_OUT_F, "%5.3lf", i);        // A

        dlgprintf(hDlg, IDC_P_F, "%5.3lf", v * i);
        if(i > 10e-3 && v > 1e-3)
          {
          r = v / i;
          rf = r - 0.097;
          t = (rf / 0.22 - 1) / 0.0046 + t_f;
          }
        else
          {
          r = 0;
          t = t_f;
          }
        dlgprintf(hDlg, IDC_R_F, "%5.3lf", r);
        dlgprintf(hDlg, IDC_T_F, "%4.0lf", t);

        t_f = C3601_GetTempearture(port, DEV_F, &errorsts);
        }

      dlgprintf(hDlg, IDC_T_CORR, "%4.1lf : %4.1lf : %4.1lf", t_a, t_e, t_f);

      if(PASSIVE_BIAS)
        {
        v = v_a - v_e;
        i = i_a - i_e;
        }
      else
        {
        v = v_a;
        i = i_a;
        }

      dlgprintf(hDlg, IDC_V_EM, "%9.3lf", v);        // V
      dlgprintf(hDlg, IDC_I_EM, "%9.3lf", i * 1e6);  // uA

      showsts();                    // show status info

      if(cycltestkey)               // Cyclic test: turn on/off with one hour quants.
        {                      // 0  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 19 20 21 22 23
        static int hrsts[24] = {  0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0 };
        time_t t;
        struct tm *tblock;

        time(&t);
        tblock = localtime(&t);

        enab_a = enab_e = enab_f = hrsts[tblock -> tm_hour];
        //enab_a = enab_e = enab_f = (tblock -> tm_min) % 4 > 0 ? 1 : 0;
        SendDlgItemMessage(hDlg, IDC_ENABLE_A, BM_SETCHECK, enab_a, 0);
        SendDlgItemMessage(hDlg, IDC_ENABLE_E, BM_SETCHECK, enab_e, 0);
        SendDlgItemMessage(hDlg, IDC_ENABLE_F, BM_SETCHECK, enab_f, 0);
        }
      else
        {
        enab_a = SendDlgItemMessage(hDlg, IDC_ENABLE_A, BM_GETCHECK, 0, 0);
        enab_e = SendDlgItemMessage(hDlg, IDC_ENABLE_E, BM_GETCHECK, 0, 0);
        enab_f = SendDlgItemMessage(hDlg, IDC_ENABLE_F, BM_GETCHECK, 0, 0);
        }

      if(use_a)
        {
        if(enab_a)
          vsout_a = vset_a;
        else
          vsout_a = 0;
        dlgprintf(hDlg, IDC_V_TO_SET_A, "%d", (int)(vsout_a + 0.5));
        pos = (vsout_a + 0.5) / vscale_a_abs * 1000;
        SendMessage(pb_a, PBM_SETPOS, pos, 0);
        C3601_SetValue(port, DEV_A, (vsout_a + voffset_a_s) / vscale_a_s, enab_a);
        }

      if(use_e)
        {

        if(PASSIVE_BIAS)
          {
          vsout_e = vset_e;
          }
        else
          {
          if(enab_e)
            vsout_e = vset_e;
          else
            vsout_e = 0;
          }

        dlgprintf(hDlg, IDC_V_TO_SET_E, "%d", (int)(vsout_e + 0.5));
        pos = (vsout_e + 0.5) / vscale_e_abs * 1000;
        SendMessage(pb_e, PBM_SETPOS, pos, 0);
        if(RESISTOR_CONTROL)
          C3601_SetValue(port, DEV_E, (vsout_e + voffset_e_s) / vscale_e_s / vscale_e_s_r, enab_e || enab_f);
        else
          C3601_SetValue(port, DEV_E, (vsout_e + voffset_e_s) / vscale_e_s, enab_e || enab_f);
        }

      if(use_f)
        {
        if(enab_f)
          vsout_f = vset_f;
        else
          vsout_f = 0;
        dlgprintf(hDlg, IDC_V_TO_SET_F, "%6.3f", vsout_f);
        pos = vsout_f / iscale_f_abs * 1000;
        SendMessage(pb_f, PBM_SETPOS, pos, 0);
        C3601_SetValue(port, DEV_F, (vsout_f + ioffset_f_s) / iscale_f_s, enab_e || enab_f);
        }

      if((enab_a || enab_e || enab_f) && (t_a >= shutdowntemp || t_f >= shutdowntemp  || t_e >= shutdowntemp || useshutdown && alarmlevel))
        {
        if(stopdealy == 0)
          {
          stopdealy = 1000 / TICK;      // one second delay
          }
        else if(--stopdealy == 0)       // delay expired and output condition is still not okay
          {
          alarm(t_a >= shutdowntemp ? acc_temp : t_f >= shutdowntemp || t_e >= shutdowntemp ? hv_temp : level);  // activate alarm dialog
          if(SendDlgItemMessage(hDlg, IDC_LOG, BM_GETCHECK, 0, 0))
            {
            logprintf("Alarm message sent: %s\n",
              t_a >= shutdowntemp ? "acc_temp" : t_f >= shutdowntemp || t_e >= shutdowntemp ? "hv_temp" : "level");
            }
          }
        }
      else stopdealy = 0;

      // log to file
      if(SendDlgItemMessage(hDlg, IDC_LOG, BM_GETCHECK, 0, 0))
        {
        if(newlog)
          {
          time_t t;
          char hdr[25];

          time(&t);
          strncpy(hdr, ctime(&t), 25);
          hdr[24] = 0;                          // clear last '\n'
          logprintf("\n%s, %d times averaged, %d seconds period\n",
                    hdr, LOGTICKS, LOGTICKS * TICK / 1000);
          vavr = iavr = tavr = 0;
          tickcnt = 0;
          newlog = 0;
          }
        else if(tickcnt < LOGTICKS)
          {
          tickcnt++;
          vavr += v_a; iavr += i_a; tavr += t_a;      // average LOGTICKS samples
          }
        else
          {

          time_t t;
          char hdr[25];

          time(&t);
          strncpy(hdr, ctime(&t), 25);
          hdr[19] = 0;              // clear last '\n'

          //logprintf("%s %6.3f %6.3f  acc=%s ext=%s fil=%s\n",   // print to log
          //          hdr + 11, tavr / LOGTICKS, vavr / LOGTICKS,
          //          enab_a ? "on" : "off", enab_e ? "on" : "off", enab_f ? "on" : "off");

          logprintf("%s  %5.2lf %9.3lf %8.3lf\n",   // print to log
                    hdr + 11, tavr / LOGTICKS, vavr / LOGTICKS, iavr * 1e6 / LOGTICKS);

          tickcnt = 0;
          vavr = iavr = tavr = 0;
          }
        }
      else
        {
        tickcnt = 0;
        newlog = 1;
        }
      break;
      }
    default: return(FALSE);
    }
  return(TRUE);
}


double detcorona(double i)
{
  static double tconst = 5, lasti, lastdi1, det;
  double nlim, plim, di1, di2, dd;

  if(i == 0)
    return det = 0;
  if(i == lasti)
    return det;

  plim = i * 0.1; nlim = -plim;

  di1 = i - lasti;
  if(di1 > 0 && lastdi1 > 0 || di1 < 0 && lastdi1 < 0)
    {
    //di1 = 0;
    di2 = 0;
    }
  else
    {
    if(di1 > plim) di1 = plim;
    else if(di1 < nlim) di1 = nlim;
    di2 = di1 - lastdi1;
    if(di2 > plim) di2 = plim;
    else if(di2 < nlim) di2 = nlim;
    }

  dd = fabs(di2) - det;
  det += dd / tconst;

  lasti = i;
  lastdi1 = di1;

  return det;
}


// show error status
void showsts(void)
{
  int sts;
  char *err, errbuf[50];
  static int errcnt;

  if(errcnt > 0)
    {
    errcnt--;
    return;
    }
  switch(errorsts)
    {
    case 0: err = "none";
         break;
    case ERR_INVALID_PORT:
         err = "Incorrect COM port number";
         break;
    case ERR_INVALID_VALUE:
         err = "Incorrect value of parameter";
         break;
    case ERR_WRITE_FAIL:
         err = "Cannot write to port";
         break;
    case ERR_READ_FAIL:
         err = "Cannot read from port";
         break;
    case ERR_READ_SUM:
         err = "Incorrect checksum";
         break;
    case ERR_INTERNAL:
         err = "Internal error";
         break;
    case ERR_ADC_OVERFLOW:
         err = "Meter overflow";
         break;
    case ERR_ADC_NEGATIVE:
         err = "Negative value from meter";
         break;
    default:
         sprintf(errbuf, "%d", errorsts);
         err = errbuf;
         break;
    }
  SetDlgItemText(MainWin, IDC_ERROR, err);
  if(errorsts)
    {
    errcnt = 1000 / TICK;
    #if(!USERMODE)
      logprintf("Error: %s\n", err);
    #endif  
    }

  sts = C3601_GetStatusFlags(port);
  if(sts >= 0)
    devicests = sts;
  /*
  sts = C3601_GetFaultFlags(port, DEV_A);
  if(sts >= 0)
    faultsts = sts;
  */
}


// filter a sample in value
void filter(double *val, double sample)
{
  #define DIFLIM     1e-4        // limit of difference
  #define MINWEIGHT  1e-2        // min. weight of each sample
  double dv = *val - sample, w;

  if(fltdelay > 0)
    {
    fltdelay--;
    *val = sample;
    return;
    }
  if(*val == 0)
    *val = sample;
  else
    {
    w = fabs(dv / *val) * 1000;
    if(w > 1)
      w = 1;
    else if(w < MINWEIGHT)
      w = MINWEIGHT;
  //if(*val == 0 || fabs(dv / *val) > DIFLIM)
  //  *val = sample;
  //else
    *val -= dv * w;
    }
}


// build i/o dependence and store to file
void curvtofile(void)
{
  FILE *f;
  char fname[256];
  double in, out, step;

  sprintf(fname, "%s\\3601vcurv.dat", workdir);
  if((f = fopen(fname, "w")) == NULL)
    return;

  pause = 1;        // suspend data refresh

  C3601_SetValue(port, DEV_A, 0, 1);
  Sleep(1000);
  step = 200;
  for(in = 0; in <= vscale_a; in += step)
    {
    C3601_SetValue(port, DEV_A, in, 1);
    Sleep(in < 1000 ? 5000 : 1000);
    out = C3601_GetVoltage(port, DEV_A, &errorsts);
    fprintf(f, "%10.1lf %10.1lf\n", in, out);
    }

  pause = 0;        // resume data refresh

  fclose(f);
}


char ininame[256];

// Read configuration data from the <test dir>\3601.ini file
void readini(void)
{
  #define CFGLIM    10000
  char buf[CFGLIM];
  double tmp;
  FILE *f;

  sprintf(ininame, "%s\\%d.ini", workdir, boardtype);
  if((f = fopen(ininame, "rb")) == NULL)
    {
    MessageBox(MainWin, "No configuration file: default values will be set.", AppName, MB_OK | MB_ICONWARNING);
    return;
    }
  fread(buf, 1, sizeof(buf), f);

  buf[CFGLIM - 1] = 0;

  readinipar(buf, "VscaleAcceleratorAbs",  &vscale_a_abs);

  readinipar(buf, "VscaleAcceleratorSet",  &vscale_a_s);
  readinipar(buf, "VoffsetAcceleratorSet", &voffset_a_s);

  readinipar(buf, "VscaleAccelerator",  &vscale_a);
  readinipar(buf, "VoffsetAccelerator", &voffset_a);
  readinipar(buf, "IscaleAccelerator",  &iscale_a);
  readinipar(buf, "IoffsetAccelerator", &ioffset_a);

  readinipar(buf, "VscaleBiasAbs",  &vscale_e_abs);
  readinipar(buf, "VscaleBiasSet",  &vscale_e_s);
  readinipar(buf, "VoffsetBiasSet", &voffset_e_s);
  readinipar(buf, "VscaleBiasSetR",  &vscale_e_s_r);

  readinipar(buf, "VscaleBias",  &vscale_e);
  readinipar(buf, "VoffsetBias", &voffset_e);
  readinipar(buf, "IscaleBias",  &iscale_e);
  readinipar(buf, "IoffsetBias", &ioffset_e);

  readinipar(buf, "IscaleFilamentAbs",  &iscale_f_abs);
  readinipar(buf, "IscaleFilamentSet",  &iscale_f_s);
  readinipar(buf, "IoffsetFilamentSet", &ioffset_f_s);

  readinipar(buf, "IscaleFilament",  &iscale_f);
  readinipar(buf, "IoffsetFilament", &ioffset_f);
  readinipar(buf, "VscaleFilament",  &vscale_f);
  readinipar(buf, "VoffsetFilament", &voffset_f);

  readinipar(buf, "ShutdownTemp", &shutdowntemp);
  readinipar(buf, "UseLevelShutdown", &tmp); useshutdown = tmp > 0.5;

  readinipar(buf, "PASSIVE_BIAS", &tmp); PASSIVE_BIAS = tmp > 0.5;
  readinipar(buf, "VOLTAGE_CONTROL_F", &tmp); VOLTAGE_CONTROL_F = tmp > 0.5;
  readinipar(buf, "EXTRA_PARAMS", &tmp); EXTRA_PARAMS = tmp > 0.5;
  readinipar(buf, "RESISTOR_CONTROL", &tmp); RESISTOR_CONTROL = tmp > 0.5;
  readinipar(buf, "SEPARATED_FEEDBACK", &tmp); SEPARATED_FEEDBACK = tmp > 0.5;

  readinipar(buf, "MinimumAcceleratorSet",  &MinimumAcceleratorSet);
  readinipar(buf, "MaximumAcceleratorSet",  &MaximumAcceleratorSet);
  readinipar(buf, "MinimumBiasSet",  &MinimumBiasSet);
  readinipar(buf, "MaximumBiasSet",  &MaximumBiasSet);
  readinipar(buf, "MinimumFilamentSet",  &MinimumFilamentSet);
  readinipar(buf, "MaximumFilamentSet",  &MaximumFilamentSet);

  MaximumAcceleratorSet = minmax(MaximumAcceleratorSet, 0, vscale_a_abs);
  MinimumAcceleratorSet = minmax(MinimumAcceleratorSet, 0, MaximumAcceleratorSet);

  MaximumBiasSet = minmax(MaximumBiasSet, 0, vscale_e_abs);
  MinimumBiasSet = minmax(MinimumBiasSet, 0, MaximumBiasSet);

  MaximumFilamentSet = minmax(MaximumFilamentSet, 0, iscale_f_abs);
  MinimumFilamentSet = minmax(MinimumFilamentSet, 0, MaximumFilamentSet);

  fclose(f);

  //vscale_a_abs = (int)((vscale_a_s + 1000) / 10000) * 10000;

}


void readinipar(char *buf, char *name, double *par)
{
  char *p;
  int len, nf;

  len = strlen(name);

  do
    {
    p = strstr(buf, name);
    if(p)
      {
      nf = sscanf(p, "%*s %lf\n", par);
      if(nf == 0)
        break;
      }
    else break;
    buf = p + len;
    }
  while(!isspace(*(p + len)));
  if(p == NULL || nf != 1)
    error("No <%s> parameter found in configuration file: default value %f will be set.", name, *par);
}



// Printf to log file
void logprintf(char *fmt, ... )
{
  va_list argptr;
  static char str[256];
  char fname[256], tstr[26];
  FILE *f;
  time_t t;

  sprintf(fname, "%s\\3601.log", workdir);
  if((f = fopen(fname, "a")) == NULL)
    return;

  va_start(argptr, fmt);
  vsprintf(str, fmt, argptr);
  va_end(argptr);
  time(&t);
  strncpy(tstr, ctime(&t), 24);
  tstr[24] = 0;
  fprintf(f, "%s %s", tstr, str);

  fclose(f);
}



// Printf to a dialog item
void dlgprintf(HWND hDlg, int id, char *fmt, ... )
{
  va_list argptr;
  static char str[256];

  va_start(argptr, fmt);
  vsprintf(str, fmt, argptr);
  SetDlgItemText(hDlg, id, str);
  va_end(argptr);
}


// Activation context generation failed for 
//"D:\svn_Cps\David_Jun_64bit\CPS_software_use this for cal\src\demo\x64\Debug\360x_Legacy.exe". 
// Dependent Assembly Microsoft.VC90.DebugCRT,processorArchitecture="amd64",publicKeyToken="1fc8b3b9a1e18e3b",type="win32",version="9.0.21022.8" could not be found. Please use sxstrace.exe for detailed diagnosis
//
// "event viewer/Summary of Administrative Events/Error/Event Id=33 SideBySide
//
//#ifdef _MSC_VER
//
///*
// * vsscanf for MSC
// *
// * Written 5/2003 by <mgix@mgix.com>
// *
// * This code is in the Public Domain
// *
// */
//
//#include <malloc.h>
//
//int vsscanf(const char  *buffer, const char  *format, va_list argPtr)
//{
//    // Get an upper bound for the # of args
//    size_t count = 0;
//    const char *p = format;
//    size_t stackSize;
//    void **newStack;
//    int result;
//    void *savedESP;
//
//    while(1)
//    {
//        char c = *(p++);
//        if(c==0) break;
//        if(c=='%' && (p[0]!='*' && p[0]!='%')) ++count;
//    }    // Make a local stack
//    stackSize = (2+count)*sizeof(void*);
//    newStack = (void**)alloca(stackSize);    // Fill local stack the way sscanf likes it
//    newStack[0] = (void*)buffer;
//    newStack[1] = (void*)format;
//    memcpy(newStack+2, argPtr, count*sizeof(void*));    // Warp into system sscanf with new stack
//    _asm
//    {
//        mov     savedESP, esp;
//        mov     esp, newStack;
//        call    sscanf;
//        mov     esp, savedESP;
//        mov     result, eax;
//    }
//    return result;
//}
//
//#endif // _MSC_VER
//
//
//// Scanf from a dialog item
//int dlgscanf(HWND hDlg, int id, char *fmt, ... )
//{
//  va_list argptr;
//  static char str[256];
//  int res;
//
//  va_start(argptr, fmt);
//  GetDlgItemText(hDlg, id, str, 255);
//  res = vsscanf(str, fmt, argptr);
//  va_end(argptr);
//  return(res);
//}

// Error message
int error(char *fmt, ... )
{
  va_list argptr;
  static char str[256];
  int res;

  //connectok = 0;
  if(debugmode)    // no error messages in debug mode
    return 0;
  va_start(argptr, fmt);
  vsprintf(str, fmt, argptr);
  res = MessageBox(0, str, AppName, MB_OK | MB_ICONEXCLAMATION);
  va_end(argptr);
  return(res);
}


#define ALARM_NAME "Alarm"

void alarm(enum alarmtype atype)
{
  if(FindWindow(0, ALARM_NAME))
    return;
  MessageBeep(MB_ICONEXCLAMATION);
  DialogBoxParam((HINSTANCE)GetModuleHandle(0), ALARM_NAME, 0, alarmdialog, (LPARAM)atype);
}


#pragma argsused

BOOL CALLBACK alarmdialog(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  static int delay;
  int cmd;
  static enum alarmtype atype;

  switch(uMsg)
    {
    case WM_INITDIALOG:
      atype = (enum alarmtype)lParam;
      delay = shutdowndelay;
      SetTimer(hDlg, 1, 1000, 0);
    case WM_TIMER:                  // timer event
      if(--delay <= 0)
        goto end;
      switch(atype)
        {
        case acc_temp:
          dlgprintf(hDlg, IDC_MSGTEXT, "Accelerator module temperature has reached critical point.\n"
                          "All outputs will be turned off after %d seonds.", delay);
          break;
        case hv_temp:
          dlgprintf(hDlg, IDC_MSGTEXT, "HV module temperature has reached critical point.\n"
                          "All outputs will be turned off after %d seonds.", delay);
          break;
        case level:
          dlgprintf(hDlg, IDC_MSGTEXT, "Coolant level has reached critical point.\n"
                          "All outputs will be turned off after %d seonds.", delay);
          break;
        }
      break;
    case WM_COMMAND:
      cmd = LOWORD(wParam);
      switch(cmd)
        {
        case IDOK:                  // close all and quit
          goto end;
        default: return(FALSE);
        }
    default: return(FALSE);
    }
  return(TRUE);
  end:
    disableall();
    KillTimer(hDlg, 1);
    EndDialog(hDlg, 0);
  return(TRUE);
}


void disableall(void)
{
  SendDlgItemMessage(MainWin, IDC_ENABLE_A, BM_SETCHECK, 0, 0);
  SendDlgItemMessage(MainWin, IDC_ENABLE_E, BM_SETCHECK, 0, 0);
  SendDlgItemMessage(MainWin, IDC_ENABLE_F, BM_SETCHECK, 0, 0);
}


void help(HELPINFO *info, HWND win)
{
  if((short) info -> iCtrlId == IDC_STATIC) return;
  if(info -> iCtrlId == 0) return;
  DialogBoxParam((HINSTANCE)GetModuleHandle(0), "Help", 0, helpdialog, (LPARAM)info);
  InvalidateRect(win, 0, 1);
  UpdateWindow(win);
}


#pragma argsused

BOOL CALLBACK helpdialog(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  HELPINFO *info;
  HDC dc;
  RECT rect;
  PAINTSTRUCT ps;
  static char *tip;
  static char buf[1024];
  static int tcnt;
  static int orgid;
  static TRACKMOUSEEVENT mevent;

  switch(uMsg)
    {
    case WM_INITDIALOG:
      {
      int x, y, sx, sy, xb, cx, cy, dx, dy;
      RECT mainrect;

      info = (HELPINFO *)lParam;
      tip = gettip(info -> iCtrlId);

      if(tip == NULL)
        goto end;

      sprintf(buf, "\n%s", tip);
      mainrect.left = 0;
      GetWindowRect(MainWin, &mainrect);

      dc = GetDC(hDlg);
      GetClientRect(hDlg, &rect);
      SelectObject(dc, GetStockObject(DEFAULT_GUI_FONT));
      DrawText(dc, buf, strlen(buf), &rect, DT_CENTER | DT_WORDBREAK | DT_CALCRECT);
      xb = 0;//GetSystemMetrics(SM_CXFIXEDFRAME);
      cx = GetSystemMetrics(SM_CXSCREEN);
      cy = GetSystemMetrics(SM_CYSCREEN);
      sx = rect.right - rect.left + xb * 2 + 60;
      sy = rect.bottom - rect.top + xb * 2 + 20;
      x = info -> MousePos.x; y = info -> MousePos.y;
      dx = cx - (x + sx + 5);
      if(dx < 0 && x + dx > mainrect.left) x += dx;
      dy = cy - (y + sy + 5);
      if(dy < 0) y += dy;
      x = max(0, x - 10);
      y = max(0, y - 2);
      MoveWindow(hDlg, x, y, sx, sy, 1);
      //SetWindowPos(hDlg, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE	| SWP_NOSIZE);
      ReleaseDC(hDlg, dc);
      orgid = info -> iCtrlId;
      tcnt = 0;
      SetTimer(hDlg, 1, 100, 0);
      break;
      }
    case WM_PAINT:
      {
      dc = BeginPaint(hDlg, &ps);
      GetClientRect(hDlg, &rect);
      FillRect(dc, &rect, GetStockObject(WHITE_BRUSH));
      SelectObject(dc, GetStockObject(DEFAULT_GUI_FONT));
      DrawText(dc, buf, strlen(buf), &rect, DT_CENTER | DT_WORDBREAK);
      EndPaint(hDlg, &ps);

      break;
      }
    case WM_MOUSELEAVE:
    case WM_LBUTTONDOWN:
    case WM_KILLFOCUS:
      goto end;
    case WM_TIMER:                  // timer event
      if(tcnt == 20)
        {
        mevent.cbSize = sizeof(mevent);
        mevent.dwFlags = TME_LEAVE;
        mevent.hwndTrack = hDlg;
        TrackMouseEvent(&mevent);
        }
      if(++tcnt > 100)
        goto end;
      break;
    default: return(FALSE);
    }
  return(TRUE);
  end:
    KillTimer(hDlg, 1);
    EndDialog(hDlg, 0);
  return(TRUE);
}


char *gettip(int id)
{
  static char buf[1024];

  switch(id)
    {
    case IDC_CPS:
      sprintf(buf, "CPS\n"
                   "7313 SW Tech Center Drive, Portland, OR 97203\n"
                   "503.684.8005\n"
                   "info@cpshv.com");
      return(buf);

    case IDC_CONSTAT_A:
      sprintf(buf, "Connection status\n for Accelerator module.");
      return(buf);
    case IDC_CONSTAT_E:
      sprintf(buf, "Connection status\n for Bias module.");
      return(buf);
    case IDC_CONSTAT_F:
      sprintf(buf, "Connection status\n for Filament module.");
      return(buf);

    case IDC_COM1:
      sprintf(buf, "Use COM1.");
      return(buf);
    case IDC_COM2:
      sprintf(buf, "Use COM2.");
      return(buf);
    case IDC_COM3:
      sprintf(buf, "Use COM3.");
      return(buf);
    case IDC_COM4:
      sprintf(buf, "Use COM4.");
      return(buf);

    case IDC_CONSTOP:
      sprintf(buf, "Stop connection.");
      return(buf);
    case IDC_CONRESTART:
      sprintf(buf, "Restart connection.");
      return(buf);
    case IDC_ENABLE_A:
      sprintf(buf, "Enable output for Accelerator.");
      return(buf);
    case IDC_V_SET_A_S:
      sprintf(buf, "Output voltage control for Accelerator\n (coarse).");
      return(buf);
    case IDC_V_SET_A_M:
    case IDC_V_SET_A_UD:
      sprintf(buf, "Output voltage control for Accelerator\n (fine).");
      return(buf);
    case IDC_V_TO_SET_A:
      sprintf(buf, "Output voltage to set for Accelerator.");
      return(buf);
    case IDC_V_OUT_A:
      sprintf(buf, "Accelerator internal voltage meter, V.");
      return(buf);
    case IDC_I_OUT_A:
      sprintf(buf, "Accelerator internal current meter, A.");
      return(buf);

    case IDC_ENABLE_E:
      sprintf(buf, "Enable output for Bias.");
      return(buf);
    case IDC_V_SET_E_S:
      if(RESISTOR_CONTROL)
        sprintf(buf, "Output resistor control for Bias\n (coarse).");
      else
        sprintf(buf, "Output voltage control for Bias\n (coarse).");
      return(buf);
    case IDC_V_SET_E_M:
    case IDC_V_SET_E_UD:
      if(RESISTOR_CONTROL)
        sprintf(buf, "Output resistor control for Bias\n (fine).");
      else
        sprintf(buf, "Output voltage control for Bias\n (fine).");
      return(buf);
    case IDC_V_TO_SET_E:
      if(RESISTOR_CONTROL)
        sprintf(buf, "Output resistor value to set for Bias.");
      else
        sprintf(buf, "Output voltage to set for Bias.");
      return(buf);
    case IDC_V_OUT_E:
      sprintf(buf, "Bias internal voltage meter, V.");
      return(buf);
    case IDC_I_OUT_E:
      sprintf(buf, "Bias internal current meter, A.");
      return(buf);
    case IDC_V_EM:
      sprintf(buf, "Emitter voltage (calculated): Accelerator minus Bias, V.");
      return(buf);
    case IDC_I_EM:
      sprintf(buf, "Wehnelt current (calculated): Accelerator minus Bias, A.");
      return(buf);
    case IDC_ENABLE_F:
      sprintf(buf, "Enable output for Filament.");
      return(buf);
    case IDC_V_SET_F_S:
      if(VOLTAGE_CONTROL_F)
        sprintf(buf, "Output voltage control for Filament\n (coarse).");
      else
        sprintf(buf, "Output current control for Filament\n (coarse).");
      return(buf);
    case IDC_V_SET_F_M:
    case IDC_V_SET_F_UD:
      if(VOLTAGE_CONTROL_F)
        sprintf(buf, "Output voltage control for Filament\n (fine).");
      else
        sprintf(buf, "Output current control for Filament\n (fine).");
      return(buf);
    case IDC_V_TO_SET_F:
      if(VOLTAGE_CONTROL_F)
        sprintf(buf, "Output voltage to set for Filament.");
      else
        sprintf(buf, "Output current to set for Filament.");
      return(buf);
    case IDC_V_OUT_F:
      sprintf(buf, "Filament internal voltage meter, V.");
      return(buf);
    case IDC_I_OUT_F:
      sprintf(buf, "Filament internal current meter, A.");
      return(buf);

    case IDC_R_F:
      sprintf(buf, "Filament resistance, Ohm.");
      return(buf);
    case IDC_P_F:
      sprintf(buf, "Filament power, W.");
      return(buf);
    case IDC_T_F:
      sprintf(buf, "Filament temperature, C.");
      return(buf);

    case IDC_ALARM:
      sprintf(buf, "Coolant level inside HV tank. Too low if RED.");
      return(buf);
    case IDC_REVISION:
      sprintf(buf, "Firmware revision\n Accelerator module : Bias module : Filament module.");
      return(buf);
    case IDC_T_CORR:
      sprintf(buf, "Internal temperature\n Accelerator module : Bias module : Filament module.");
      return(buf);
    case IDC_ERROR:
      sprintf(buf, "Error status.");
      return(buf);
    case IDC_LOG:
      sprintf(buf, "Log data to the file \"3601.log\".");
      return(buf);
    case IDC_CLOSE:
      sprintf(buf, "Turn device off and close application. Use <Ctrl> to reload .ini file.");
      return(buf);
    
    default: return 0;
    }

}



